use std::cell::RefCell;
use std::ops::Deref;

use crate::{game::*, APP_ID};

use gtk::glib::clone;
use gtk::prelude::*;
use glib::subclass::InitializingObject;
use gtk::gio::Settings;
use gtk::subclass::prelude::*;
use gtk::{glib, Label, Entry, CompositeTemplate};
use rand::Rng;


#[derive(CompositeTemplate, Default)]
#[template(resource = "/org/foxarmy/learn-hieroglyph/guessing/game/ui.xml")]
pub struct GuessingScene {
    #[template_child]
    pub question_label: TemplateChild<Label>,
    #[template_child]
    pub answer_entry: TemplateChild<Entry>,
    #[template_child]
    pub stats_label: TemplateChild<Label>
}

#[glib::object_subclass]
impl ObjectSubclass for GuessingScene {
    const NAME: &'static str = "GuessingScene";
    type Type = super::GuessingScene;
    type ParentType = gtk::ApplicationWindow;

    fn class_init(klass: &mut Self::Class) {
        klass.bind_template();
    }

    fn instance_init(obj: &InitializingObject<Self>) {
        obj.init_template();
    }
}

impl ObjectImpl for GuessingScene {
    fn constructed(&self) {
        self.parent_constructed();

        let settings = Settings::new(APP_ID);

        let ktr: bool = settings.boolean("is-ktr-enabled");
        let rtk: bool = settings.boolean("is-rtk-enabled");
        let hiragana: bool = settings.boolean("is-hiragana-enabled");
        let katakana: bool = settings.boolean("is-katakana-enabled");

        let question: RefCell<Hieroglyph> = RefCell::new(ask(ktr, rtk, hiragana, katakana));
        let exact_kana: RefCell<Kanas> = RefCell::new(if hiragana && katakana {
            if rand::thread_rng().gen_bool(0.5) {
                Kanas::Hiragana
            } else {
                Kanas::Katakana
            }
        } else if hiragana {
            Kanas::Hiragana
        } else {
            Kanas::Katakana
        });

        let correct_answer: RefCell<Hieroglyph> = RefCell::new(get_kana_pair_for_hieroglyph(
            match question.borrow().get_kana() {
                Kanas::Romaji => match exact_kana.borrow().deref() {
                    Kanas::Hiragana => &Kanas::Hiragana,
                    Kanas::Katakana => &Kanas::Katakana,
                    _ => panic!("HOW DID YOU GET HERE?!"),
                },
                _ => &Kanas::Romaji,
            },
            &question.borrow(),
        ));

        let binding = generate_question_text(&question.borrow(), exact_kana.borrow().deref());
        let text: &str = &(binding.as_str());
        self.question_label.set_label(text);

        let stats: RefCell<(u32, u32)> = RefCell::new((0, 0));

        let question_label_binding = &self.question_label.get();
        let answer_entry_binding = &self.answer_entry.get();
        let stats_label_binding = &self.stats_label.get();

        let iteration = clone!(@strong stats, 
                                                @strong question_label_binding as question_label, 
                                                @strong answer_entry_binding as answer_entry, 
                                                @strong stats_label_binding as stats_label => move |_entry: &Entry| {
            let answer: &String = &answer_entry.text().to_string();
            println!("{} <-> {}? = {}", answer, correct_answer.borrow().to_string(), *answer == correct_answer.borrow().to_string());
            *stats.borrow_mut() = if *answer == correct_answer.borrow().to_string() { (stats.borrow().0 + 1, stats.borrow().1) } else {(stats.borrow().0, stats.borrow().1 + 1)};
            stats_label.set_label(format!("Correct|Incorrect: {}|{} ({:.2}%)", stats.borrow().0, stats.borrow().1, 100.0 * f64::from(stats.borrow().0) / f64::from(stats.borrow().0 + stats.borrow().1)).as_str());
            *question.borrow_mut() = ask(ktr, rtk, hiragana, katakana);
            *exact_kana.borrow_mut() =
                if hiragana && katakana {
                    if rand::thread_rng().gen_bool(0.5) {Kanas::Hiragana} else {Kanas::Katakana}
                } else if hiragana {
                    Kanas::Hiragana
                } else {
                    Kanas::Katakana
                };
            *correct_answer.borrow_mut() = get_kana_pair_for_hieroglyph(
                match question.borrow().get_kana() {
                    Kanas::Romaji => match exact_kana.borrow().deref() {
                        Kanas::Hiragana => &Kanas::Hiragana,
                        Kanas::Katakana => &Kanas::Katakana,
                        _ => panic!("HOW DID YOU GET HERE?!")
                    },
                    _ => &Kanas::Romaji
                },
                &question.borrow()
            );
            let binding = generate_question_text(&question.borrow(), exact_kana.borrow().deref());
            let text: &str = &(binding.as_str());
            question_label.set_label(text);
            answer_entry.set_text("");
        });

        self.answer_entry.connect_activate(iteration);

    }
}

impl WidgetImpl for GuessingScene {}

impl WindowImpl for GuessingScene {}

impl ApplicationWindowImpl for GuessingScene {}